/********************************************************************************************************
                                               uC/OS-III
                                         The Real-Time Kernel

                                          CSKY Specific code

 File : os_cpu_a.S
********************************************************************************************************/
#include <csi_config.h>

.file     "os_cpu_a.S"

#undef  VIC_TSPDR
#define VIC_TSPDR 0XE000EC08

.import  OSTimeTick
.import  OSIntExit
.import  OSTaskSwHook

.import  OSTCBCurPtr
.import  OSTCBHighRdyPtr
.import  OSPrioCur
.import  OSPrioHighRdy
.import  OSRunning
.import  OSIntNestingCtr

.text
.align 2

/********************************************************************
 * Functions:
 *     size_t cpu_sr_save(void);
 ********************************************************************/

.global cpu_sr_save
.type   cpu_sr_save, %function
cpu_sr_save:
    mfcr    r0, psr
    psrclr  ie
    rts

/********************************************************************
 * Functions:
 *     void cpu_sr_restore(size_t psr);
 ********************************************************************/

.global cpu_sr_restore
.type   cpu_sr_restore, %function
cpu_sr_restore:
    mtcr    r0, psr
    rts

/********************************************************************
 *  Procedure : OSStartHighRdy
 *
 *  The procedure gets the highest priority task then loads its
 *  stack before restoring all of the registers and doing an RTE
 *  to actually start the task. Called directly from uC/OS
 ********************************************************************/

.global OSStartHighRdy
.type   OSStartHighRdy, %function
OSStartHighRdy:
    psrclr  ie
    jbr     __tspend_handler_nosave

/********************************************************************
 * Functions:
 *     void OS_TASK_SW(void);
 ********************************************************************/

.global OS_TASK_SW
.type   OS_TASK_SW, %function
OS_TASK_SW:
    lrw     r0, VIC_TSPDR
    bgeni   r1, 0
    stw     r1, (r0)
    rts

/********************************************************************
 *  Procedure : OSIntCtxSw
 *
 *  The procedure is called by when the uC/OS wants to return to
 *  a different task at interrupt time. Used primarily within the
 *  timer tick interrupt.
 ********************************************************************/

.global OSIntCtxSw
.type   OSIntCtxSw, %function
OSIntCtxSw:
    lrw     r0, VIC_TSPDR
    bgeni   r1, 0
    stw     r1, (r0)
    rts

.global tspend_handler
.type   tspend_handler, %function
tspend_handler:
    subi    sp, 132
    stm     r0-r13, (sp)
    stw     r15, (sp, 56)
    addi    r0, sp, 60
    fstms   vr0-vr15, (r0)
    mfcr    r1, epsr
    stw     r1, (r0, 64)
    mfcr    r1, epc
    stw     r1, (r0, 68)

    lrw     r0, OSTCBCurPtr
    ldw     r1, (r0)
    stw     sp, (r1)

__tspend_handler_nosave:
    lrw     r6, OSTCBHighRdyPtr
    lrw     r7, OSTCBCurPtr
    ldw     r8, (r6)
    stw     r8, (r7)

#ifdef CONFIG_STACK_GUARD
    jbsr    csky_set_stackbound
#endif

    ldw     sp, (r8)

#ifdef CONFIG_STACK_GUARD
    mfcr    r3, cr<0, 4>
    bseti   r3, 0
    bseti   r3, 1
    mtcr    r3, cr<0, 4>
#endif

    jbsr    OSTaskSwHook

    ldw     r0, (sp, 128)
    mtcr    r0, epc
    ldw     r0, (sp, 124)
    mtcr    r0, epsr

    ldm     r0-r13, (sp)
    ldw     r15, (sp, 56)
    addi    sp, 60
    fldms   vr0-vr15, (sp)
    addi    sp, 72

    rte
