/*
 * Copyright (C) 2017 C-SKY Microsystems Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/******************************************************************************
 * @file     ckmgu.h
 * @brief    this file contains the functions support mgu operations.
 * @version  V1.0
 * @date     20. July 2016
 ******************************************************************************/

#ifndef _CKMGU_H_
#define _CKMGU_H_

/* defines of region size */
#define MGU_REGION_4K   0x16
#define MGU_REGION_8K   0x18
#define MGU_REGION_16K  0x1a
#define MGU_REGION_32K  0x1c
#define MGU_REGION_64K  0x1e
#define MGU_REGION_128K 0x20
#define MGU_REGION_256K 0x22
#define MGU_REGION_512K 0x24
#define MGU_REGION_1M   0x26
#define MGU_REGION_2M   0x28
#define MGU_REGION_4M   0x2a
#define MGU_REGION_8M   0x2c
#define MGU_REGION_16M  0x2e
#define MGU_REGION_32M  0x30
#define MGU_REGION_64M  0x32
#define MGU_REGION_128M 0x34
#define MGU_REGION_256M 0x36
#define MGU_REGION_512M 0x38
#define MGU_REGION_1G   0x3a
#define MGU_REGION_2G   0x3c
#define MGU_REGION_4G   0x3e

/* defines of region permission */
#define MGU_REGION_RW               0x300     /* user RW & super RW */
#define MGU_REGION_RO               0x200     /* user RO & super RW */
#define MGU_REGION_NOTACCESSIBLE    0x100     /* user not accessible & super RW */
#define MGU_REGION_SNOTACCESSIBLE   0x000     /* both user and super not accessible */

/* defines of region cacheable */
#define MGU_REGION_UNCACHE          0x0
#define MGU_REGION_CACHE            0x1

.macro  CSKY_MGU_CONFIG region addr len permission cache
/* set the mgu permission and cache */
mfcr    r4, cr19
lrw     r7, \permission
movi    r6, \cache
movi    r5, \region
lsl     r6, r5
lsli    r5, 1
lsl     r7, r5
or      r7, r6
or      r7, r4
mtcr    r7, cr19

/* set region */
movi    r5, \region
mtcr    r5, cr21
movi    r5, \len
bseti   r5, 0
lrw     r7, \addr
or      r7, r5
mtcr    r7, cr20

.endm

.macro  CSKY_MGU_ENABLE
/* MGU enable */
mfcr    r7, cr18
bseti   r7, 0
mtcr    r7, cr18
.endm

.macro  CSKY_MGU_DISABLE
/* MGU enable */
mfcr    r7, cr18
bclri   r7, 0
mtcr    r7, cr18
.endm

#endif /* _CKMGU_H_ */
