/*
 * Copyright (C) 2017 C-SKY Microsystems Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/******************************************************************************
 * @file     ckmpu.h
 * @brief    this file contains the functions support mpu operations.
 * @version  V1.0
 * @date     20. July 2016
 ******************************************************************************/

#ifndef _CKMPU_H_
#define _CKMPU_H_

/* defines of region size */
#define MPU_REGION_1K   0x12
#define MPU_REGION_2K   0x14
#define MPU_REGION_4K   0x16
#define MPU_REGION_8K   0x18
#define MPU_REGION_16K  0x1a
#define MPU_REGION_32K  0x1c
#define MPU_REGION_64K  0x1e
#define MPU_REGION_128K 0x20
#define MPU_REGION_256K 0x22
#define MPU_REGION_512K 0x24
#define MPU_REGION_1M   0x26
#define MPU_REGION_2M   0x28
#define MPU_REGION_4M   0x2a
#define MPU_REGION_8M   0x2c
#define MPU_REGION_16M  0x2e
#define MPU_REGION_32M  0x30
#define MPU_REGION_64M  0x32
#define MPU_REGION_128M 0x34
#define MPU_REGION_256M 0x36
#define MPU_REGION_512M 0x38
#define MPU_REGION_1G   0x3a
#define MPU_REGION_2G   0x3c
#define MPU_REGION_4G   0x3e

/* defines of region permission */
#define MPU_REGION_RW               0x300     /* user RW & super RW */
#define MPU_REGION_RO               0x200     /* user RO & super RW */
#define MPU_REGION_NOTACCESSIBLE    0x100     /* user not accessible & super RW */
#define MPU_REGION_SNOTACCESSIBLE   0x000     /* both user and super not accessible */

/* defines of region executable */
#define MPU_REGION_X          0x0
#define MPU_REGION_NX         0x1

#define MPU_REGION_UNSAVE     0x0
#define MPU_REGION_SAVE       0x10000000

.macro  CSKY_MPU_CONFIG region addr len permission executable security
/* set the MPU permission, executable and security */
mfcr    r4, cr19
lrw     r7, \permission
movi    r6, \executable
movi    r5, \region
lsl     r6, r5
lsli    r5, 1
lsl     r7, r5
lrw     r3, \security
lsli    r3, \region
or      r7, r6
or      r7, r4
or      r7, r3
mtcr    r7, cr19

/* set region */
movi    r5, \region
mtcr    r5, cr21
movi    r5, \len
bseti   r5, 0
lrw     r7, \addr
or      r7, r5
mtcr    r7, cr20

.endm

.macro  CSKY_MPU_ENABLE
/* MPU enable */
mfcr    r7, cr18
bseti   r7, 0
mtcr    r7, cr18
.endm

.macro  CSKY_MPU_DISABLE
/* MPU enable */
mfcr    r7, cr18
bclri   r7, 0
mtcr    r7, cr18
.endm

#endif /* _CKMPU_H_ */
