/*
 * Copyright (C) 2017 C-SKY Microsystems Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/******************************************************************************
 * @file     irq.h
 * @brief    this file contains the functions support irq operations.
 * @version  V1.0
 * @date     20. July 2016
 ******************************************************************************/

#ifndef __INCLUDE_CSKY_IRQ_H
#define __INCLUDE_CSKY_IRQ_H

/* Save the current primask state & disable IRQs */
static inline unsigned long irqsave(void)
{
    unsigned long flags;
    asm volatile(
        "mfcr   %0, psr  \n"
        "psrclr ie      \n"
        :"=r"(flags) : :"memory");
    return flags;
}

/* Restore saved primask state */

static inline void irqrestore(unsigned long flags)
{
    asm volatile(
        "mtcr    %0, psr  \n"
        : :"r"(flags) :"memory");
}

#ifndef __ASSEMBLY__
#ifdef __cplusplus
#define EXTERN extern "C"
extern "C"
{
#else
#define EXTERN extern
#endif

#undef EXTERN
#ifdef __cplusplus
}
#endif
#endif

#endif /* __INCLUDE_CSKY_IRQ_H */

